@@ -1,4 +1,5 @@
 import base64
+import functools
 import time
 from uuid import uuid4
 
@@ -8,6 +9,7 @@ from urllib3.exceptions import NewConnectionError, TimeoutError
 from hoss_agent.conf.constants import EVENT
 from hoss_agent.context import execution_context
 from hoss_agent.utils.buffer_proxy import BufferProxy
+from hoss_agent.utils.compat import BytesIO
 from hoss_agent.utils.logging import get_logger
 
 logger = get_logger("hoss_agent.instrumentaion.packages.urllib3_hooks")
@@ -22,9 +24,15 @@ def install(module):
         module,
         "response.HTTPResponse.read", read,
     )
+
+    wrapt.wrap_function_wrapper(
+        module, "response.HTTPResponse.stream", _response_stream
+    )
+
     wrapt.wrap_function_wrapper(
         module,
-        "response.HTTPResponse.read_chunked", read_chunked,
+        "response.HTTPResponse.release_conn",
+        _response_release_conn,
     )
 
 
@@ -70,12 +78,18 @@ def _capture_error(connection, method, scheme, host, url, body, headers, request
                 },
                 "receivedAt": int(round(time.time() * 1000))
             }
+        req_body = BytesIO()
+        if body is not None:
+            if isinstance(body, bytes):
+                req_body.write(body)
+            elif isinstance(body, str):
+                req_body.write(body.encode("utf-8"))
         event = {
             "eventId": uuid4(),
             "request": {
                 "method": method,
                 "headers": dict(headers),
-                "body": base64.b64encode(body) if body is not None else "",
+                "body": req_body,
                 "url": "%s://%s%s" % (
                     scheme,
                     host,
@@ -90,16 +104,50 @@ def _capture_error(connection, method, scheme, host, url, body, headers, request
         logger.debug('Error capturing urllib.HTTPConnectionPool._make_request', ex)
 
 
+def _response_release_conn(wrapped, instance, args, kwargs):
+    rv = wrapped(*args, **kwargs)
+    try:
+        response = getattr(instance, "_original_response")
+        hoss_context = getattr(response, "_hoss_context", None)
+        if not hoss_context:
+            return rv
+
+        if 'queued' not in hoss_context['event']:
+            hoss_context['event']['queued'] = True
+            hoss_context['client'].queue(EVENT, hoss_context['event'])
+
+    except Exception as ex:
+        pass
+    return rv
+
+
+def _response_stream(wrapped, instance, args, kwargs):
+    response = getattr(instance, "_original_response")
+    hoss_context = getattr(response, "_hoss_context", None)
+
+    if not hoss_context:
+        return wrapped(*args, **kwargs)
+
+    stream = wrapped(*args, **kwargs)
+    proxy = BufferProxy(stream, hoss_context['event']['response']['body'])
+
+    hoss_context['response_proxy'] = True
+
+    return proxy
+
+
 # this hooks into urllib3.response
 def read(wrapped, instance, args, kwargs):
     rv = wrapped(*args, **kwargs)
     try:
-        hoss_context = getattr(instance._fp, "_hoss_context", None)
+        hoss_context = getattr(instance._original_response, "_hoss_context", None)
         if not hoss_context:
             return rv
-        hoss_context['event']['response']['body'].write(rv)
+        if not hoss_context.get('response_proxy'):
+            hoss_context['event']['response']['body'].write(rv)
+
         if instance.closed:
-            logger.debug('Response is cloed after read. Queue event')
+            logger.debug('Response is closed after read. Queue event')
             if 'queued' not in hoss_context['event']:
                 hoss_context['event']['queued'] = True
                 hoss_context['client'].queue(EVENT, hoss_context['event'])
@@ -108,18 +156,3 @@ def read(wrapped, instance, args, kwargs):
 
     return rv
 
-
-# this is called to read an amt bytes out of the response. if hoss context doesn't have
-# a response body yet, set it with the return value, otherwise concatenate
-def read_chunked(wrapped, instance, args, kwargs):
-    rv = wrapped(*args, **kwargs)
-    try:
-        hoss_context = getattr(instance._fp, "_hoss_context", None)
-        if hoss_context is None:
-            return rv
-
-        rv = BufferProxy(rv,  hoss_context['event']['response']['body'])
-    except Exception as ex:
-        logger.debug('Error in monkey-patched urllib3.response.read_chunked: %s', ex)
-    return rv
-
