@@ -182,8 +182,8 @@ class Transport(ThreadManager):
         try:
             if isinstance(data['request']['body'], bytes):
                 data['request']['body'] = data['request']['body'].decode()
-            elif isinstance(data['request']['body'], BytesIO):
-                data['request']['body'] = data['request']['body'].getvalue().decode()
+            elif hasattr(data['request']['body'], 'getvalue'):
+                data['request']['body'] = data['request']['body'].getvalue()
             else:
                 logger.debug('Cannot decode request body into string: %s. Removing it')
                 data['request']['body'] = None
@@ -194,8 +194,8 @@ class Transport(ThreadManager):
             if 'response' in data:
                 if isinstance(data['response']['body'], bytes):
                     data['response']['body'] = data['response']['body'].decode()
-                elif isinstance(data['response']['body'], BytesIO):
-                    data['response']['body'] = data['response']['body'].getvalue().decode()
+                elif hasattr(data['response']['body'], 'getvalue'):
+                    data['response']['body'] = data['response']['body'].getvalue()
                 else:
                     logger.debug('Cannot decode response body into string: %s. Removing it')
                     data['response']['body'] = None
