@@ -5,7 +5,6 @@ def get_runtime_env():
     return {
         "arch": platform.machine(),
         "hostname": platform.node() or "unknown",
-        "type": platform.system(),
-        "platform:": platform.platform(),
+        "platform:": platform.system(),
         "version": platform.python_version(),
     }
