@@ -7,6 +7,7 @@ import urllib3
 from urllib3.exceptions import ConnectionError, TimeoutError, NewConnectionError
 
 import hoss_agent
+
 try:
     from unittest.mock import MagicMock, patch, ANY
 except ImportError:
@@ -74,6 +75,22 @@ class TestUrllib3(unittest.TestCase):
         self.assertEqual(event['response']['statusCode'], 200)
         self.assertEqual(event['response']['headers'], dict(response.headers))
 
+    def test_post_with_bytes_body(self, start_threads, transport_start_threads, queue):
+        request_id = str(uuid4())
+        url = "https://postman-echo.com/post?id=" + request_id
+
+        http = urllib3.PoolManager()
+        response = http.request('POST',
+                                url,
+                                body=b'test',
+                                headers={'Content-Type': 'application/json'})
+        queue.assert_called_once_with(EVENT, ANY, False)
+
+        event = queue.call_args_list[0][0][1]
+
+        self.assertEqual(event['request']['body'].getvalue(), b'test')
+
+
     def test_chunked(self, start_threads, transport_start_threads, queue):
         url = "https://postman-echo.com/stream/1"
         http = urllib3.PoolManager()
@@ -153,3 +170,52 @@ class TestUrllib3(unittest.TestCase):
 
         self.assertFalse('response' in event)
         self.assertEqual(event['error']['type'], 'CONNECTION_ERROR')
+
+    def test_connection_error_post(self, start_threads, transport_start_threads, queue):
+        url = "https://fasdftasdfa-echo.com/delay/2"
+        try:
+            http = urllib3.PoolManager()
+            http.request('POST', url, retries=False,
+                         body=json.dumps({'key': 'value'}).encode('utf-8'),
+                         headers={'Content-Type': 'application/json'})
+        except NewConnectionError as ex:
+            pass
+
+        queue.assert_called_once_with(EVENT, ANY, False)
+        event_type = queue.call_args_list[0][0][0]
+        event = queue.call_args_list[0][0][1]
+        self.assertEqual(event_type, EVENT)
+
+        self.assertFalse('response' in event)
+        self.assertEqual(event['error']['type'], 'CONNECTION_ERROR')
+        self.assertEqual(event['request']['body'].getvalue(), b'{"key": "value"}')
+
+    def test_file(self, start_threads, transport_start_threads, queue):
+        url = "https://fasdftasdfa-echo.com/delay/2"
+        try:
+            http = urllib3.PoolManager()
+            http.request('GET', url, retries=False)
+        except NewConnectionError as ex:
+            pass
+
+        queue.assert_called_once_with(EVENT, ANY, False)
+        event_type = queue.call_args_list[0][0][0]
+        event = queue.call_args_list[0][0][1]
+        self.assertEqual(event_type, EVENT)
+
+        self.assertFalse('response' in event)
+        self.assertEqual(event['error']['type'], 'CONNECTION_ERROR')
+
+    def test_file(self, start_threads, transport_start_threads, queue):
+        url = "http://httpbin.org/post"
+        http = urllib3.PoolManager()
+        with open('tests/instrumentations/test.txt', 'rb') as fp:
+            file_data = fp.read()
+            http.request('POST', url, fields={
+                'filefield': ('test.txt', file_data , 'text/plain'),
+            })
+        event = queue.call_args_list[0][0][1]
+        self.assertTrue(event['request']['headers']['Content-Type'].startswith('multipart/form-data; boundary='))
+        body = event['request']['body'].getvalue()
+        # content of test.txt
+        self.assertTrue('foobar' in body.decode())
